# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from django.template import Context, Template

from django.contrib.sites.models import Site

from bursary.models import Bursary

SUBJECT_TEMPLATE = Template(
    '[Action needed] Reimbursement process for {{ WAFER_CONFERENCE_NAME }} '
    'travel bursary recipients')

BODY_TEMPLATE = Template("""\
{% autoescape off %}
Dear {{ object.user.get_full_name }},

You have been granted a travel bursary for DebConf, and now is the time to
submit documentation for reimbursement. TL;DR: yes, this email is very long,
but it is all important, you need to read it in full to get travel money back!

Reimbursement for DebConf-related expenses will be processed by  Debian France
for reimbursements in EUR (typically Pan-European travelers) and Software in
the Public Interest (SPI) for US and other non-EUR reimbursment requests.

The amount you requested is {{ object.travel_bursary }} {{ DEBCONF_BURSARY_CURRENCY }}, and this is therefore the maximum you'll
be able to request for reimbursement.
This does not have to be requested *in* {{ DEBCONF_BURSARY_CURRENCY }}, you can request an equivalent
amount in your home currency (via SPI).

To process the hundreds of reimbursement requests efficiently and
expeditiously, our supporting organizations require complete, readable and
quality controlled submissions, so please take a moment extra to double check
you have everything in order before sending off that final email.

For ̲D̲e̲b̲i̲a̲n̲ ̲F̲r̲a̲n̲c̲e (EUR reimbursements, typically flights, trains and ferries):

Please scan your receipts and attach them to an email to
treasurer-debianfr@rt.debian.org.
You ̲m̲u̲s̲t include "Debian RT - {{ WAFER_CONFERENCE_NAME }} reimbursement for your full name"
in the subject line.
Put tresorier@france.debian.net in CC.

Put the following information into your email:

1. Your full legal name
2. Your bank account details (account owner, IBAN and BIC, ...)
3. Digital copies of your receipts
4. The following text:

--- BEGIN TICKET TEXT ---
By submitting this reimbursement request, I declare:
 • that I have attached sufficient documentation substantiating my request,
 • that I seek reimbursement of expenses that are compliant to DebConf policies, and
 • that I have not sought nor will seek reimbursement of these expenses from any other source.

The amount I am claiming for reimbursement is XXX.XX EUR (out of {{ object.travel_bursary }} {{ DEBCONF_BURSARY_CURRENCY }} allocated).
---- END TICKET TEXT ----

(adjust the reimbursement total so it matches the total of your receipts)

For ̲S̲P̲I (non-EUR reimbursements, typically cross-continental travel):

All requests must include:

 1. an SPI Reimbursement Request Form [1] with accurate banking information as
    SPI uses TransferWise [2]
 2. an Expense Report generated using XE's Travel Expenses Calculator [3]
 3. sufficient documentation substantiating the expense report
 4. a declaration of compliance

And must be submitted to the debconf-reimbursements RT queue (instead of the
normal SPI reimbursement RT queue, listed on [1]).

[1]: https://spi-inc.org/treasurer/reimbursement-form/
[2]: https://transferwise.com
[3]: https://www.xe.com/travel-expenses-calculator/

Requesters must follow these steps:

Step 1: Prepare an Expense Report.
 • use https://www.xe.com/travel-expenses-calculator/ to prepare an expense
   report
 • enter Your Name
 • set Your Home Currency to the currency of your bank account.
 • leave/set Credit Card @ 3%, Debit Card @ 5%, Foreign Cash @ 5%, Traveler's
   Cheque @ 2%
 • enter Receipt Details, one row per receipt
   • specify the correct date of the transactions
   • set the currency per transaction to the currency you paid in
 • save as PDF

Step 2: Prepare an SPI Reimbursement Request Form
 • visit https://spi-inc.org/treasurer/reimbursement-form/
 • fill out the form and download the completed PDF
 • ensure your name exactly matches the Expense Report from Step 1
 • ̲e̲n̲s̲u̲r̲e̲ ̲t̲h̲a̲t̲ ̲t̲h̲e̲ ̲b̲a̲n̲k̲i̲n̲g̲ ̲i̲n̲f̲o̲r̲m̲a̲t̲i̲o̲n̲ ̲i̲s̲ ̲a̲c̲c̲u̲r̲a̲t̲e̲ ̲a̲s̲ ̲i̲n̲c̲o̲r̲r̲e̲c̲t̲ ̲d̲e̲t̲a̲i̲l̲s̲ ̲a̲r̲e̲ ̲a
   ̲m̲a̲j̲o̲r̲ ̲s̲o̲u̲r̲c̲e̲ ̲o̲f̲ ̲d̲e̲l̲a̲y̲s
 • the currency field is the currency of your bank account. Only say {{ DEBCONF_BURSARY_CURRENCY }}
   here if you have a {{ DEBCONF_BURSARY_CURRENCY }} bank account (e.g. because you live there)
 • save as PDF

Step 3: Collect and order your Receipts.
 • collect your receipts in the SAME ORDER as the rows in the Expense Report
 • if paper receipts, scan them with a multi-function device, converting to PDF
 • save as PDF

Step 4: Prepare the Submission Package.
 • collect into a single PDF, the following:
   • from step 2, the SPI Reimbursement Request From
   • from step 1, the Expense Report
   • from step 3, the Ordered Receipts
 • the poppler-utils package includes the pdfunite utility
   • usage: `pdfunite step2.pdf step1.pdf step3a.pdf step3b.pdf ... step3n.pdf {{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf`
   • warning: ensure that you explicitly mention the destination filename
     ({{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf)
     otherwise step3n.pdf is overwritten
   • notice: please name the file exactly as shown, substituting your name as
     entered in the SPI Reimbursement Request Form, for example,
     {{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf
 • ̲e̲n̲s̲u̲r̲e̲ ̲t̲h̲a̲t̲ ̲t̲h̲e̲ ̲e̲n̲t̲i̲r̲e̲ ̲S̲u̲b̲m̲i̲s̲s̲i̲o̲n̲ ̲P̲a̲c̲k̲a̲g̲e̲ ̲c̲a̲n̲ ̲b̲e̲ ̲e̲a̲s̲i̲l̲y̲ ̲u̲n̲d̲e̲r̲s̲t̲o̲o̲d̲ ̲a̲s̲ ̲p̲o̲o̲r
   ̲q̲u̲a̲l̲i̲t̲y̲ ̲s̲u̲b̲m̲i̲s̲s̲i̲o̲n̲s̲ ̲a̲r̲e̲ ̲a̲ ̲m̲a̲j̲o̲r̲ ̲s̲o̲u̲r̲c̲e̲ ̲o̲f̲ ̲d̲e̲l̲a̲y̲s

Step 5: Email the Submission Package.
 • prepare an email having these attributes
   • from: you
   • To: debconf-reimbursements@rt.spi-inc.org
   • subject: {{ WAFER_CONFERENCE_NAME }} Reimbursement Request for {{ object.user.get_full_name }}
   • attachment: the Submission Package ({{ WAFER_CONFERENCE_NAME }}ReimbursementRequest-{{ short_full_name }}.pdf) prepared in step 4
   • body: (the text below, please fix the claimed amount in the last line to
            match your actual request, in your local currency)
--- BEGIN TICKET TEXT ---
By submitting this reimbursement request, I declare:
 • that I have accurately prepared an SPI Reimbursement Request Form,
 • that I have prepared an Expense Report using XE's Travel Expense Calculator,
 • that I have attached sufficient documentation substantiating my request,
 • that I seek reimbursement of expenses that are compliant to DebConf policies, and
 • that I have not sought nor will seek reimbursement of these expenses from any other source.

The amount I am claiming for reimbursement is XXXX.XX your local currency or USD (out of {{ object.travel_bursary }} {{ DEBCONF_BURSARY_CURRENCY }} allocated).
---- END TICKET TEXT ----

If you have any question, please be sure to contact us at bursaries@debconf.org
and we'll work your issues out.

Thanks for your cooperation!
--\u0020
The DebConf Bursaries team
{% endautoescape %}
""")


class Command(BaseCommand):
    help = 'Send an email to people whose bursary grant expires soon'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually send emails')

    def badger(self, bursary, conference_name, dry_run):
        context = {
            'object': bursary,
            'user': bursary.user.username,
            'to': '%s <%s>' % (bursary.user.get_full_name(),
                               bursary.user.email),
            'short_full_name': bursary.user.get_full_name().replace(' ', '')
                                                           .replace('-', '')
                                                           .replace("'", '')
                                                           .replace('.', ''),
            'DEBCONF_BURSARY_CURRENCY': settings.DEBCONF_BURSARY_CURRENCY,
            'WAFER_CONFERENCE_NAME': conference_name,
        }

        if dry_run:
            print('I would badger {to} (max = {object.travel_bursary})'
                  .format(**context))
            return

        from_email = 'bursaries@debconf.org'
        ctx = Context(context)
        subject = SUBJECT_TEMPLATE.render(ctx)
        body = BODY_TEMPLATE.render(ctx)

        msg = EmailMultiAlternatives(subject, body, to=[context['to']],
                                     from_email=from_email)

        msg.send()

    def handle(self, *args, **options):
        site = Site.objects.get()
        conference_name = site.name.replace(' ', '')
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        to_badger = Bursary.objects.filter(
            request_travel=True,
            travel_status='accepted',
            user__attendee__check_in__pk__isnull=False,
            reimbursed_amount=0,
        ).order_by('user__username')

        for bursary in to_badger:
            self.badger(bursary, conference_name, dry_run)



# We have no models, but some model-related behaviour

from collections import OrderedDict

import debconf.talk_urls
from wafer.schedule.admin import SCHEDULE_ITEM_VALIDATORS

# Workaround strictness of wafer schedule validation
non_contiguous = None
for i, validator in enumerate(SCHEDULE_ITEM_VALIDATORS):
    _, err_type, _ = validator
    if err_type == "non_contiguous":
        non_contiguous = i
        break
if non_contiguous is not None:
    SCHEDULE_ITEM_VALIDATORS.pop(non_contiguous)

GENDERS = OrderedDict((
    ('', 'Decline to state'),
    ('m', 'Male'),
    ('f', 'Female'),
    ('n', 'Non-Binary'),
))



from django.conf import settings
from django.core.signing import Signer
from django.core.validators import MinValueValidator
from django.db import models
from django.urls import reverse


class Invoice(models.Model):
    STATUS_CHOICES = (
        ('new', 'Invoiced'),
        ('pending', 'Payment pending'),
        ('paid', 'Payment received'),
        ('canceled', 'Invoice canceled'),
        ('disputed', 'Payment disputed'),
        ('refunded', 'Payment refunded'),
    )

    reference_number = models.CharField(max_length=128, unique=True,
                                        null=False, blank=False)
    status = models.CharField(max_length=128, choices=STATUS_CHOICES,
                              default='new')
    date = models.DateField()

    recipient = models.ForeignKey(settings.AUTH_USER_MODEL,
                                  related_name='invoices',
                                  on_delete=models.PROTECT)
    invoiced_entity = models.CharField(max_length=128, blank=True)
    billing_address = models.TextField()
    compound = models.BooleanField(default=False)

    transaction_id = models.CharField(max_length=128, null=False, blank=True)

    created = models.DateTimeField(auto_now_add=True)
    last_update = models.DateTimeField(auto_now=True)

    signer = Signer(sep=':', salt='invoices.Invoice')

    @property
    def total(self):
        return sum(line.total for line in self.lines.all())

    @property
    def total_local(self):
        return round(self.total * settings.DEBCONF_LOCAL_CURRENCY_RATE, 2)

    def text_details(self):
        header = ('Reference', 'Description', 'Qty', 'Unit', 'Total')
        footer = ('', '', '', 'Total', str(self.total))
        lines = [
            (line.reference, line.description, line.quantity, line.unit_price,
             line.total)
            for line in self.lines.all()
        ]
        all_lines = (
            [
                header,
                ('', '', '', '', ''),
            ]
            + lines
            + [
                ('', '', '', '-------', '-------'),
                footer,
            ]
        )

        col_width = [max(len(str(x)) for x in col) for col in zip(*all_lines)]
        formats = ['{:{}}'] * 3 + ['{:>{}}'] * 2
        return "\n".join(
            ("| " + " | ".join(formats[i].format(x, col_width[i])
                               for i, x in enumerate(line)) + " |")
            for line in all_lines
        )

    def save(self, *args, **kwargs):
        # generate reference number on save
        if not self.reference_number:
            year = str(self.date.year)
            last_invoice = Invoice.objects.filter(
                reference_number__startswith=year
            ).order_by('-reference_number').first()

            if last_invoice:
                year, seqnum = last_invoice.reference_number.split('-')
                seqnum = int(seqnum, 10) + 1
            else:
                seqnum = 1

            self.reference_number = '%s-%05d' % (year, seqnum)

        return super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse(
            'invoices:display',
            kwargs={'reference_number': self.reference_number})

    def get_signed_url(self):
        signed_reference = self.signer.sign(self.reference_number)
        return reverse(
            'invoices:display',
            kwargs={'reference_number': signed_reference})

    def get_payment_intent(self):
        # FIXME: Replace with better column
        from invoices.stripe_payments import payment_intent
        return payment_intent(self)

    def income_breakdown(self):
        summary = {}
        for line in self.lines.all():
            if line.reference.startswith('INV#'):
                # There are two types of existing invoices:
                # Combined invoices have a positive sign
                # Previous payments have a negative sign
                sign = 1 if line.total >= 0 else -1
                ref = line.reference.split('#', 1)[1]
                try:
                    sub_invoice = Invoice.objects.get(reference_number=ref)
                except Invoice.DoesNotExist:
                    continue
                for category, value in sub_invoice.income_breakdown().items():
                    summary[category] = summary.get(category, 0) + value * sign
            else:
                category = line.breakdown_category
                summary[category] = summary.get(category, 0) + line.total
        return summary


class InvoiceLine(models.Model):
    invoice = models.ForeignKey(Invoice, related_name='lines',
                                on_delete=models.CASCADE)
    line_order = models.IntegerField()
    reference = models.CharField(max_length=32)
    description = models.CharField(max_length=1024)
    unit_price = models.DecimalField(max_digits=8, decimal_places=2)
    quantity = models.IntegerField(validators=[MinValueValidator(1)])

    @property
    def total(self):
        return self.unit_price * self.quantity

    class Meta:
        unique_together = ('invoice', 'line_order')
        ordering = ('invoice', 'line_order')

    def __str__(self):
        return 'InvoiceLine(%s [%s] %d @ %.02f = %.02f)' % (
            self.reference,
            self.description,
            self.quantity,
            self.unit_price,
            self.total,
        )

    @property
    def breakdown_category(self):
        ref = self.reference
        if ref.startswith(settings.INVOICE_PREFIX):
            ref = ref[len(settings.INVOICE_PREFIX):].split('-')[0]
        return {
            'REG': 'registration',
            'ACCOMM': 'accommodation',
            'BREAKFAST': 'meals',
            'LUNCH': 'meals',
            'DINNER': 'meals',
            'CONFDINNER': 'meals',
        }.get(ref, 'registration')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-07-16 12:50
from __future__ import unicode_literals

from django.db import migrations

from register.views import STEPS


def backfill_registration_timestamps(apps, schema_editor):
    last_step = len(STEPS) - 1
    Attendee = apps.get_model('register', 'Attendee')

    for attendee in Attendee.objects.all():
        user = attendee.user
        update = {
            'created_timestamp': user.date_joined,
            'updated_timestamp': user.date_joined,
        }
        if attendee.completed_register_steps == last_step:
            update['completed_timestamp'] = user.date_joined
        # Avoid using .save() so we can set an old updated_timestamp
        Attendee.objects.filter(id=attendee.id).update(**update)


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0007_add_registration_timestamps'),
    ]

    operations = [
        migrations.RunPython(backfill_registration_timestamps, noop),
    ]



# -*- coding: utf-8 -*-

from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from django.template import Context, Template

from django.contrib.sites.models import Site

from register.models import Attendee

SUBJECT_TEMPLATE = Template('Feedback for {{ WAFER_CONFERENCE_NAME }}')

BODY_TEMPLATE = Template("""\
{% autoescape off %}
Dear {{ attendee.user.get_full_name }},

We hope you're all adjusting back to your day-to-day post-DebConf lives, and
that your travels back were safe and uneventful.

We'd like to hear your thoughts about this DebConf: was there anything you
particularly liked? Or perhaps something you really think should have been done
differently?

DebConf values your feedback! Please get in touch with us at:
<feedback@debconf.org>

Both suggestions for improvement and positive feedback are very welcome, and
will be anonymised, then passed along to the relevant teams.

Best regards,

The {{ WAFER_CONFERENCE_NAME }} Team
{% endautoescape %}
""")


class Command(BaseCommand):
    help = 'Send a feedback request email to attendees who arrived'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually send emails')

    def badger(self, attendee, conference_name, dry_run):
        context = {
            'attendee': attendee,
            'to': '%s <%s>' % (attendee.user.get_full_name(),
                               attendee.user.email),
            'WAFER_CONFERENCE_NAME': conference_name,
        }

        if dry_run:
            print('I would badger {to}'.format(**context))
            return

        from_email = 'feedback@debconf.org'
        ctx = Context(context)
        subject = SUBJECT_TEMPLATE.render(ctx)
        body = BODY_TEMPLATE.render(ctx)

        msg = EmailMultiAlternatives(subject, body, to=[context['to']],
                                     from_email=from_email)

        msg.send()

    def handle(self, *args, **options):
        site = Site.objects.get()
        conference_name = site.name.replace(' ', '')
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        to_badger = Attendee.objects.filter(
            check_in__pk__isnull=False,
        ).order_by('user__username')

        for attendee in to_badger:
            self.badger(attendee, conference_name, dry_run)



import datetime

from django.db import models
from django.conf import settings

from register.dates import get_ranges_for_dates
from register.models.attendee import Attendee


PAID_ACCOMMODATION = any(
    'price' in option for option in settings.PRICES['accomm'].values())


def accomm_option_choices(bursary=False, paid=False):
    for key, details in settings.PRICES['accomm'].items():
        if bursary and not details.get('bursary', False):
            continue
        if paid and not details.get('price', None):
            continue

        description = details['description']
        if 'price' in details:
            description += (
                f" ({details['price']} {settings.DEBCONF_BILLING_CURRENCY}"
                f"/night)")
        yield (key, description)


class AccommNight(models.Model):
    date = models.DateField(unique=True)

    @property
    def form_name(self):
        return 'night_{}'.format(self)

    def __str__(self):
        return self.date.isoformat()

    class Meta:
        ordering = ['date']


class Accomm(models.Model):
    attendee = models.OneToOneField(Attendee, related_name='accomm',
                                    on_delete=models.CASCADE)
    nights = models.ManyToManyField(AccommNight)
    option = models.CharField(max_length=32,
                              choices=accomm_option_choices())
    requirements = models.TextField(blank=True)
    family_usernames = models.TextField(blank=True)
    room = models.CharField(max_length=128, blank=True, default='')

    def __str__(self):
        return 'Accomm <{}>'.format(self.attendee.user.username)

    def get_checkin_checkouts(self):
        """Get the successive check-in and check-out dates for the attendee"""
        stays = get_ranges_for_dates(
            night.date for night in self.nights.all()
        )

        for first_night, last_night in stays:
            yield first_night
            yield last_night + datetime.timedelta(days=1)

    def get_stay_details(self):
        """Get the check-in, check-out for each stay"""
        ci_co = iter(self.get_checkin_checkouts())
        return zip(ci_co, ci_co)

    def get_roommates(self):
        if self.room:
            return Attendee.objects.filter(
                accomm__room=self.room).exclude(id=self.attendee_id)



# Generated by Django 2.2.12 on 2020-05-30 18:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('volunteers', '0002_remove_volunteer_tasks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='volunteers.TaskCategory'),
        ),
        migrations.AlterField(
            model_name='tasktemplate',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='volunteers.TaskCategory'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand

from invoices.models import Invoice
from invoices.stripe_payments import PaymentIntent, build_metadata


class Command(BaseCommand):
    help = 'Update the breakdown metedata on invoices, in Stripe'

    def update_invoice(self, invoice):
        PaymentIntent.modify(invoice.transaction_id,
                             metadata=build_metadata(invoice))

    def handle(self, *args, **options):
        for invoice in Invoice.objects.all().exclude(transaction_id=''):
            self.update_invoice(invoice)



from django.conf import settings
from django.forms import Form
from django.urls import reverse

from register.models.attendee import Attendee
from register.views.core import RegisterStep


class ConfirmationView(RegisterStep):
    title = 'Confirmation'
    form_class = Form
    template_name = 'register/page/confirmation.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        try:
            context['attendee'] = self.request.user.attendee
        except Attendee.DoesNotExist:
            pass

        context['DEBCONF_CITY'] = settings.DEBCONF_CITY

        return context

    def get_success_url(self):
        return reverse('wafer_user_profile',
                       kwargs={'username': self.request.user.username})



from django.contrib import admin

from front_desk.models import CheckIn


class CheckInAdmin(admin.ModelAdmin):
    list_display = (
        'attendee', 'full_name', 'email', 'swag', 'nametag',
        'room_key', 'returned_key', 'checked_out',
    )
    list_filter = (
        'swag', 't_shirt', 'shoes', 'nametag', 'room_key', 'returned_key',
        'checked_out',
    )
    search_fields = (
        'attendee__user__username', 'attendee__user__first_name',
        'attendee__user__last_name', 'key_card',
    )

    def full_name(self, instance):
        return instance.attendee.user.get_full_name()
    full_name.admin_order_field = 'attendee__user__last_name'

    def email(self, instance):
        return instance.attendee.user.email
    email.admin_order_field = 'attendee__user__email'


admin.site.register(CheckIn, CheckInAdmin)



from django.urls import re_path
from django.views.generic.base import RedirectView

from register.views import STEPS
from register.views.arrived import DCScheduleArrived
from register.views.core import ClosedView, UnRegisterView
from register.views.statistics import StatisticsView


urlpatterns = [
    re_path(r'^$', RedirectView.as_view(url='step-0'), name='register'),
    re_path(r'^unregister$', UnRegisterView.as_view(), name='unregister'),
    re_path(r'^closed$', ClosedView.as_view(), name='register-closed'),
    re_path(r'^statistics/$', StatisticsView.as_view(), name='register-statistics'),
    re_path(r'^attendees/admin/export/arrived/$', DCScheduleArrived.as_view()),
]

for i, step in enumerate(STEPS):
    urlpatterns.append(
        re_path(r'^step-{}$'.format(i), step.as_view(),
            name='register-step-{}'.format(i)))



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Fieldset, Layout


class ChildCareForm(forms.Form):
    child_care = forms.BooleanField(
        label='I need child-care for my kid(s)',
        required=False,
    )
    needs = forms.CharField(
        label='The child care services I need are',
        help_text='How many hours a day? All the conference or only part of '
                  'it? etc.',
        widget=forms.Textarea(attrs={'rows': 3}),
        required=False,
    )
    details = forms.CharField(
        label='Important information about my kid(s)',
        help_text='Number, ages, languages spoken, special needs, etc.',
        widget=forms.Textarea(attrs={'rows': 5}),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False

        self.helper.layout = Layout(
            Field('child_care', id='child_care'),
            Fieldset(
                'Childcare Details',
                'needs',
                'details',
                css_id='details',
            ),
        )

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.get('child_care'):
            if not cleaned_data.get('needs'):
                self.add_error('needs', 'Please provide us with your needs.')
            if not cleaned_data.get('details'):
                self.add_error(
                    'details',
                    "Please provide us with your children's details.")

        return cleaned_data



# -*- coding: utf-8 -*-
from collections import namedtuple

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.template import Context, engines

from dc17.models import Accomm, Attendee, Food


SUBJECT = "Welcome to DebConf17! (arrival details)"

TEMPLATE = """\
Dear {{ name }},

Welcome to DebConf17, hosted at the Collège de Maisonneuve in Montreal, Canada.
We're delighted that you decided to join us for what we hope will be one of the
best DebConfs yet.

Please read the contents of this mail, and the attached PDF, carefully, as it
contains important information related to your travel and visit to Montreal.

== Before departing for Montreal ==

Print this e-mail and the attached PDF which also includes a venue map.

In case of any problems (including delays or cancellation), please contact
<registration@debconf.org> as soon as possible, or if you are already
travelling, phone or SMS the Front Desk at +1 514-316-7065 (use this number
sparingly after-hours Eastern time).

Check your data, and report any errors and changes as soon as possible to
<registration@debconf.org>:

Arriving: {{ attendee.arrival|date:"DATETIME_FORMAT" }}
Departing: {{ attendee.departure|date:"DATETIME_FORMAT" }}{% if food %}
Meals requested:{% for day in meals %}
{{ day.date|date:"SHORT_DATE_FORMAT" }}: {{ day.meals|join:", "|title }}{% endfor %}
Diet: {{ food.diet|default:"Whatever is provided" }}{% if food.special_diet %}
Details: {{ food.special_diet }}{% endif %}{% else %}
No conference-organised food requested.{% endif %}

Familiarize yourself with the Codes of Conduct for both Debian [1] and DebConf
[2].  If you are harassed, or observe harassment, you can contact
<antiharassment@debian.org> or members of the local anti-harassment team who
are recognizable with pink conference badges.  For any other kind of problem,
the Front Desk and Organisation team may be of help.

[1] http://debconf.org/codeofconduct.shtml
[2] https://www.debian.org/code_of_conduct

Check your travel documents.  Will your passport remain valid for the duration
of your stay?  Do you have all necessary travel authorizations such as an eTA
or visa?  If are used to visiting Canada without a visa and haven't in the last
6 months, you may be unaware of the new eTA system.  You should check if you
need an eTA, and apply online ASAP:

https://travel.gc.ca/returning/customs/entering-canada

Notify your bank that you'll be travelling to Canada to avoid having your
card(s) blocked.

Do you have travel insurance?  This could be important should you become in
need of medical attention, as the Canadian public health insurance system does
not cover travellers.  Be sure to also check with your provider if you're
covered in case of lost or stolen items, such as electronics and computer
devices.

Note that cheese may be imported but duty may be charged if the total product
value exceeds 20 CAD.  For alcohol, the permitted allowance is 1,5L of wine and
up to 8,5L of beer.  This information might be relevant to your (optional)
contribution to the Cheese and Wine party.

== Your Accommodation Details ==
{% if not accomm %}
You have not requested any conference-provided accommodation.  Remember to
print the details of your accommodation in case you're asked by immigration
officials.{% endif %}{% if stays.onsite %}

=== On-site accommodation ===

According to our records, you're registered to stay on-site on the following
days:

Check-in: {{ stays.onsite.checkin|date }}
Checkout: {{ stays.onsite.checkout|date }}
Room: {{ accomm.onsite_room }}

No bed sheets, blanket, or duvet are provided, only a cot and pillow.  You will
need to bring a sleeping bag, blanket and pillowcase.  Lockers will be
available on the venue to store your valuables, so bring also a padlock or
combination lock.  Don't forget to bring any showering items you might need:
bath towel, facecloth, soap, shampoo and the like.  You'll be sharing a room
with others, so if you're sensitive to noise when sleeping, consider bringing
earplugs.

https://wiki.debconf.org/wiki/DebConf17/Accommodation#On-site{% endif %}{% if stays.rvc %}

=== McGill Royal Victoria College Residence ===

According to our records, you're registered at RVC on the following days:

Check-in: {{ stays.rvc.checkin|date }}
Checkout: {{ stays.rvc.checkout|date }}
Room: {{ accomm.rvc_room }}

Bed sheets and a pillow are provided, as are showering necessities such as a
towel, soap and shampoo.

https://wiki.debconf.org/wiki/DebConf17/Accommodation#McGill_Royal_Victoria_College_Residence{% endif %}{% if stays.hotel %}

=== Hôtel Universel Montréal ===

According to our records, you're registered at the Hotel on the following days:

Check-in: {{ stays.hotel.checkin|date }}
Checkout: {{ stays.hotel.checkout|date }}

Refer to the hotel front desk for details.

https://wiki.debconf.org/wiki/DebConf17/Accommodation#Hotel_Universel{% endif %}

== What to Bring ==

An international adapter (if required) and power supplies for all your devices.

Light clothes.  It's summer in Montreal, and the days are warm and often humid.
The daily maximums range from 22 to 30°C, while nights usually drop to around
15 to 18°C.  The conference venue will be air conditioned so depending on the
outdoor conditions and the number of people in the venue, the indoor
temperature could get relatively cold so do also bring along a couple of warmer
clothing items such as a hoodie.  It will rain from time to time so a raincoat
and/or umbrella will be useful.  Don't forget a reusable water bootle and
sunscreen if you plan to enjoy the outdoors.

An Ethernet cable is always useful to have, if we have WiFi trouble.  WiFi
should be available in every area we use on the campus.

Printed PGP/GPG fingerprint slips, if you want to have your key signed by other
peers.

== Airport Arrival ==

=== Public Transit (747 bus) ===

See page 3 of the attached PDF.

=== Taxis ===

Upon exiting the border control check zone and arriving in the part of the
airport open to the public, look for the taxi signs.  Follow them outside and
the taxis should be there.

The taxi fare from the airport to downtown Montreal is priced at a fixed 40
CAD.  It might cost you more to get to the venue, since it is about 5km east of
the downtown area.  You can simply hail for a cab at the airport.  Uber is also
available.

Ask them to drop you off at Collège de Maisonneuve.  The address is 3800
Sherbrooke Est.  Once inside, look for the signs showing you the way to the
front desk!

== Front Desk location ==

The Front Desk is located on the second floor, in the main hall.  Look for a
windowed room with DebConf signs.  If you're arriving late (after 22:00) and
Front Desk is closed, there will be a volunteer stationed at the Nicolet
entrance who will help you get into your designated dorm.  If you're arriving
very late (after 3am) you'll need to go through the 3800 Sherbrooke entrance
and use the buzzer to call a security guard to open the door for you.  In case
of problems, the Front Desk may be reached at +1 514-316-7065.

Have a safe trip.  We look forward to seeing you in Montréal.

The DebConf Team
"""

Stay = namedtuple('Stay', ('checkin', 'checkout'))
Meal = namedtuple('Meal', ('date', 'meals'))


def meal_sort(meal):
    return ['breakfast', 'lunch', 'dinner'].index(meal)


class Command(BaseCommand):
    help = 'Send welcome emails'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),
        parser.add_argument('--username',
                            help='Send mail to a specific user, only'),

    def badger(self, attendee, dry_run):
        name = attendee.user.userprofile.display_name()
        to = attendee.user.email

        try:
            accomm = attendee.accomm
        except Accomm.DoesNotExist:
            accomm = None

        try:
            food = attendee.food
        except Food.DoesNotExist:
            food = None

        stays = {}
        if accomm:
            stay_details = attendee.accomm.get_stay_details()
            for checkin, checkout, alt_accomm in stay_details:
                venue = {
                    '': 'onsite',
                    'hotel': 'hotel',
                    'rvc_single': 'rvc',
                    'rvc_double': 'rvc',
                }[alt_accomm]
                stays[venue] = Stay(checkin, checkout)

        meals = []
        if food:
            by_day = {}
            for meal in food.meals.all():
                by_day.setdefault(meal.date, []).append(meal.meal)
            for date, days_meals in sorted(by_day.items()):
                meals.append(Meal(date, sorted(days_meals, key=meal_sort)))

        ctx = {
            'accomm': accomm,
            'attendee': attendee,
            'food': food,
            'meals': meals,
            'name': name,
            'stays': stays,
        }

        template = engines['django'].from_string(TEMPLATE)
        body = template.render(Context(ctx))

        if dry_run:
            print('Would badger %s <%s>' % (name, to))
            return

        attachments = []
        if (settings.EMAIL_BACKEND !=
                'django.core.mail.backends.console.EmailBackend'):
            attachments.append(
                ('DebConf17-arrival.pdf', self.attachment, 'application/pdf'))

        email_message = EmailMultiAlternatives(
            SUBJECT, body, to=["%s <%s>" % (name, to)],
            attachments=attachments)
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        with open('assets/docs/welcome.pdf', 'rb') as f:
            self.attachment = f.read()

        queryset = Attendee.objects.filter(reconfirm=True)
        if options['username']:
            queryset = queryset.filter(user__username=options['username'])

        for attendee in queryset:
            self.badger(attendee, dry_run)



from wafer.management.commands.wafer_add_default_groups import Command as AddGroupsCommand


class Command(AddGroupsCommand):
    help = "Add Django groups for DebConf teams"

    GROUPS = {
        # Permissions are specified as (app, code_name) pairs
        'Bursary Admins': (
            ('bursary', 'add_bursaryreferee'),
            ('bursary', 'change_bursaryreferee'),
            ('bursary', 'add_bursary'),
            ('bursary', 'change_bursary'),
        ),
        'Bursary Referees': (
            ('bursary', 'change_bursaryreferee'),
        ),
        'Content Admin': (
            ('auth', 'view_user'),
            ('schedule', 'add_day'),
            ('schedule', 'add_scheduleitem'),
            ('schedule', 'add_slot'),
            ('schedule', 'add_venue'),
            ('schedule', 'change_day'),
            ('schedule', 'change_scheduleitem'),
            ('schedule', 'change_slot'),
            ('schedule', 'change_venue'),
            ('schedule', 'delete_day'),
            ('schedule', 'delete_scheduleitem'),
            ('schedule', 'delete_slot'),
            ('schedule', 'delete_venue'),
            ('talks', 'add_review'),
            ('talks', 'add_reviewaspect'),
            ('talks', 'add_score'),
            ('talks', 'add_talk'),
            ('talks', 'add_talktype'),
            ('talks', 'add_talkurl'),
            ('talks', 'add_track'),
            ('talks', 'change_review'),
            ('talks', 'change_reviewaspect'),
            ('talks', 'change_score'),
            ('talks', 'change_talk'),
            ('talks', 'change_talktype'),
            ('talks', 'change_talkurl'),
            ('talks', 'change_track'),
            ('talks', 'delete_review'),
            ('talks', 'delete_reviewaspect'),
            ('talks', 'delete_score'),
            ('talks', 'delete_talktype'),
            ('talks', 'delete_talkurl'),
            ('talks', 'delete_track'),
            ('talks', 'edit_private_notes'),
            ('talks', 'view_all_talks'),
        ),
        'Registration': (
            ('front_desk', 'add_checkin'),
            ('front_desk', 'change_checkin'),
            ('register', 'change_accomm'),
            ('register', 'change_attendee'),
            ('register', 'change_food'),
        ),
        'Registration Admin': (
            ('auth', 'add_user'),
            ('auth', 'change_user'),
            ('auth', 'delete_user'),
            ('front_desk', 'add_checkin'),
            ('front_desk', 'change_checkin'),
            ('front_desk', 'delete_checkin'),
            ('invoices', 'change_invoice'),
            ('register', 'add_accomm'),
            ('register', 'add_attendee'),
            ('register', 'add_food'),
            ('register', 'change_accomm'),
            ('register', 'change_attendee'),
            ('register', 'change_food'),
            ('register', 'delete_accomm'),
            ('register', 'delete_attendee'),
            ('register', 'delete_food'),
            ('users', 'add_userprofile'),
            ('users', 'change_userprofile'),
            ('users', 'delete_userprofile'),
        ),
        'Video Team': (
            ('talks', 'view_all_talks'),
            ('volunteers', 'accept_video_tasks'),
        ),
        'Volunteer Admin': (
            ('volunteers', 'add_task'),
            ('volunteers', 'add_taskcategory'),
            ('volunteers', 'add_tasktemplate'),
            ('volunteers', 'add_volunteer'),
            ('volunteers', 'change_task'),
            ('volunteers', 'change_taskcategory'),
            ('volunteers', 'change_tasktemplate'),
            ('volunteers', 'change_volunteer'),
            ('volunteers', 'delete_task'),
            ('volunteers', 'delete_taskcategory'),
            ('volunteers', 'delete_tasktemplate'),
            ('volunteers', 'delete_volunteer'),
        ),
    }



import pytest


@pytest.mark.usefixtures("disable_caching")
class TestStatistics:
    def test_renders(self, client, create_talk, user, attendee):
        from wafer.talks.models import Review
        talk_1 = create_talk(status='A', username=attendee.user.username)
        talk_2 = create_talk(status='A')
        Review.objects.create(talk=talk_2, reviewer=user, notes='Foo')
        response = client.get('/talks/statistics/')
        assert response.status_code == 200

        assert response.context['talks_submitted'] == 2
        assert response.context['talks_reviewed'] == 1
        assert response.context['talks_scheduled'] == 0
        assert response.context['hours_of_content'] == 0
        assert response.context['hours_of_concurrency'] == []
        assert response.context['talks_by_track'] == {}
        assert response.context['talks_by_type'] == {}
        assert response.context['speakers_by_country'] == [
            ('Canada', 1), ('Not registered yet', 1)]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-04 19:33
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('schedule', '0007_venue_add_video'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=1024, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('nbr_volunteers_min', models.IntegerField(blank=True, default=1, null=True)),
                ('nbr_volunteers_max', models.IntegerField(blank=True, default=1, null=True)),
                ('start', models.DateTimeField(blank=True)),
                ('end', models.DateTimeField(blank=True)),
            ],
            options={
                'ordering': ['start', '-end', 'schedule_item', 'template__name', 'name'],
            },
        ),
        migrations.CreateModel(
            name='TaskCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=1024)),
                ('description', models.TextField()),
            ],
            options={
                'verbose_name': 'task category',
                'verbose_name_plural': 'task categories',
            },
        ),
        migrations.CreateModel(
            name='TaskTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=1024, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('nbr_volunteers_min', models.IntegerField(blank=True, default=1, null=True)),
                ('nbr_volunteers_max', models.IntegerField(blank=True, default=1, null=True)),
                ('video_task', models.BooleanField(default=False)),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='volunteers.TaskCategory')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Volunteer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('staff_rating', models.IntegerField(blank=True, choices=[(0, 'No longer welcome'), (1, 'Poor'), (2, 'Not great'), (3, 'Average'), (4, 'Good'), (5, 'Superb')], null=True)),
                ('staff_notes', models.TextField(blank=True, null=True)),
                ('preferred_categories', models.ManyToManyField(blank=True, to='volunteers.TaskCategory')),
                ('preferred_task_types', models.ManyToManyField(blank=True, to='volunteers.TaskTemplate')),
                ('tasks', models.ManyToManyField(blank=True, to='volunteers.Task')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='volunteer', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='task',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='volunteers.TaskCategory'),
        ),
        migrations.AddField(
            model_name='task',
            name='schedule_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='schedule.ScheduleItem'),
        ),
        migrations.AddField(
            model_name='task',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='volunteers.TaskTemplate'),
        ),
        migrations.AddField(
            model_name='task',
            name='venue',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='schedule.Venue'),
        ),
        migrations.AddField(
            model_name='task',
            name='volunteers',
            field=models.ManyToManyField(blank=True, to='volunteers.Volunteer'),
        ),
    ]



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Layout

from bursary.models import Bursary
from invoices.prices import meal_prices, meal_price_string
from register.dates import conference_dinner_meal, meal_choices
from register.models.food import Food
from register.fields import MealSelectionField


class FoodForm(forms.Form):
    meals = forms.MultipleChoiceField(
        label='I want to eat catered food for these meals:',
        choices=meal_choices(),
        widget=forms.CheckboxSelectMultiple,
        help_text="If you don't have a food bursary, meal prices are: {}"
                  .format(meal_price_string()),
        required=False,
    )
    conference_dinner = forms.BooleanField(
        label='I want to attend the conference dinner',
        help_text="If you don't have a food bursary, the conference dinner "
                  "will cost {conference_dinner} {currency}."
                  .format(**meal_prices()),
        required=False,
    )
    diet = forms.ChoiceField(
        label='My diet',
        choices=Food.DIETS.items(),
        required=False,
    )
    special_diet = forms.CharField(
        label='Details of my special dietary needs',
        required=False,
    )

    def __init__(self, *args, **kwargs):
        attendee = kwargs.pop('attendee')
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False
        conference_dinner = ()
        if conference_dinner_meal():
            conference_dinner = (Field('conference_dinner'),)
        self.helper.layout = Layout(
            MealSelectionField('meals', id='meals'),
            *conference_dinner,
            Field('diet', id='diet'),
            Field('special_diet', id='special_diet'),
        )

        try:
            self.bursary = attendee.user.bursary
        except Bursary.DoesNotExist:
            self.bursary = Bursary()

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.pop('conference_dinner'):
            cleaned_data['meals'].append(conference_dinner_meal())

        if self.bursary.request_food and not cleaned_data.get('meals'):
            self.add_error(
                'meals', 'Food bursary was requested, but no meals selected.')

        if (cleaned_data.get('diet') == 'other' and
                not cleaned_data.get('special_diet')):
            self.add_error('special_diet', 'Required when diet is "other"')



from django import forms
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy as _p

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit

from minidebconf.models import Diet, Registration, RegistrationType, ShirtSize


def register_form_factory():
    form_fields = ['full_name']
    if settings.MINIDEBCONF_REGISTER_PHONE is not None:
        form_fields.append('phone_number')
    if RegistrationType.objects.exists():
        form_fields.append('registration_type')
    form_fields.append('full_name')
    form_fields.append('involvement')
    form_fields.append('gender')
    form_fields.append('country')
    form_fields.append('city_state')
    form_fields.append('days')
    if Diet.objects.exists():
        form_fields.append('diet')
    if ShirtSize.objects.exists():
        form_fields.append('shirt_size')


    class RegisterForm(forms.ModelForm):
        full_name = forms.CharField(max_length=256, label=_('Full name'))
        class Meta:
            model = Registration
            fields = form_fields
            widgets = {
                'days': forms.CheckboxSelectMultiple(),
            }

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.fields["full_name"].initial = self.instance.full_name
            if 'phone_number' in self.fields:
                self.fields["phone_number"].required = settings.MINIDEBCONF_REGISTER_PHONE
            self.helper = FormHelper()

            if self.instance.id:
                submit = _p("conference", "Update registration")
            else:
                submit = _p("conference", "Register")
            self.helper.add_input(Submit("submit", submit))


        def save(self):
            super().save()
            name = self.cleaned_data['full_name'].split()
            user = self.instance.user
            user.first_name = name[0]
            user.last_name = " ".join(name[1:])
            user.save()
    return RegisterForm



from django.urls import re_path

from front_desk.views import (
    CashInvoicePayment, ChangeFoodView, ChangeShirtView, ChangeShoesView,
    CheckInView, CheckOutView, Dashboard, RegisterOnSite, RoomsView)


urlpatterns = [
    re_path(r'^$', Dashboard.as_view(), name='front_desk'),
    re_path(r'^check_in/(?P<username>[\w.@+-]+)/$', CheckInView.as_view(),
        name='front_desk.check_in'),
    re_path(r'^check_in/(?P<username>[\w.@+-]+)/change_shirt/$',
        ChangeShirtView.as_view(), name='front_desk.change_shirt'),
    re_path(r'^check_in/(?P<username>[\w.@+-]+)/change_shoes/$',
        ChangeShoesView.as_view(), name='front_desk.change_shoes'),
    re_path(r'^check_in/(?P<username>[\w.@+-]+)/change_food/$',
        ChangeFoodView.as_view(), name='front_desk.change_food'),
    re_path(r'^register/$', RegisterOnSite.as_view(), name='front_desk.register'),
    re_path(r'^cash_payment/invoice/(?P<ref>[^/]+)/$',
        CashInvoicePayment.as_view(), name='front_desk.cash_invoice_payment'),
    re_path(r'^check_out/(?P<username>[\w.@+-]+)/$', CheckOutView.as_view(),
        name='front_desk.check_out'),
    re_path(r'^rooms$', RoomsView.as_view(), name='front_desk.rooms'),
]



# -*- coding: utf-8 -*-
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from dc17.models import Accomm

ACCOMM_CHOICE_LABELS = {
    '': 'On-site dormitories',
    'rvc_single': 'Single room at McGill residences',
    'rvc_double': 'Double room at McGill residences',
    'hotel': 'Room at the Hôtel Universel',
}

SUBJECT = "Check-in and check-out dates for your DebConf 17 accommodation"

TEMPLATE = """\
Dear {name},

DebCamp and DebConf are approaching fast, and we have finalized the details
regarding accommodation during the conference. Here is what our reservation
records show for you:

{accommodation_details}

As a reminder, here are the details for the different accommodation options:
https://wiki.debconf.org/wiki/DebConf17/Accommodation

Cheers,
The DebConf Registration Team
"""

ON_SITE_MESSAGE = """\
Room assignations for the on-site dormitories are still in progress. We will
let you know as soon as the sorting hat decides on your roommates."""


def ascii_table(data, formats):
    col_width = [max(len(str(x)) for x in col) for col in zip(*data)]
    return "\n".join(
        ("| " + " | ".join(formats[i].format(x, col_width[i])
                           for i, x in enumerate(line)) + " |")
        for line in data
    )


class Command(BaseCommand):
    help = 'Badger users for confirmation'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),

    def badger(self, accomm, dry_run):
        name = accomm.attendee.user.get_full_name()
        to = accomm.attendee.user.email
        stays = list(accomm.get_stay_details())

        fmt = '%a %d %B %Y'

        stay_details = [
            (ACCOMM_CHOICE_LABELS.get(choice, 'On-site accommodation'),
             checkin.strftime(fmt), checkout.strftime(fmt))
            for checkin, checkout, choice in stays
        ]

        has_on_site = any(not(choice) for _, _, choice in stays)

        if dry_run:
            print('Would badger %s <%s> (%s) (%s)' %
                  (name, to, stay_details, has_on_site))
            return

        table_contents = [
            ('', 'Check-in', 'Check-out'),
        ] + stay_details

        accommodation_details = ascii_table(table_contents,
                                            ["{:{}}", "{:^{}}", "{:^{}}"])
        if has_on_site:
            accommodation_details += '\n\n%s' % ON_SITE_MESSAGE

        body = TEMPLATE.format(
            name=name,
            accommodation_details=accommodation_details,
        )

        email_message = EmailMultiAlternatives(SUBJECT, body,
                                               to=["%s <%s>" % (name, to)])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        for accomm in Accomm.objects.filter(attendee__reconfirm=True):
            self.badger(accomm, dry_run)



from django.urls import re_path

from badges.views import OwnBadge, CheckInBadgeView

urlpatterns = [
    re_path(r'^$', OwnBadge.as_view(),
        name='badges.own'),
    re_path(r'^check_in/(?P<username>[\w.@+-]+)/$', CheckInBadgeView.as_view(),
        name='badges.check_in'),
]



from register.dates import (
    ARRIVE_ON_OR_AFTER, ORGA_ARRIVE_ON_OR_AFTER, LEAVE_ON_OR_BEFORE
)
from register.forms.conference import ConferenceRegistrationForm
from register.models.attendee import Attendee
from register.views.core import RegisterStep


class ConferenceRegistrationView(RegisterStep):
    template_name = 'register/page/conference.html'
    title = 'Conference Registration'
    form_class = ConferenceRegistrationForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.request.user.is_staff:
            context['ARRIVE_ON_OR_AFTER'] = ORGA_ARRIVE_ON_OR_AFTER
        else:
            context['ARRIVE_ON_OR_AFTER'] = ARRIVE_ON_OR_AFTER
        context['LEAVE_ON_OR_BEFORE'] = LEAVE_ON_OR_BEFORE
        return context

    def get_initial(self):
        user = self.request.user
        initial = {}

        attendee = user.attendee
        for field in attendee._meta.get_fields():
            if field.is_relation:
                continue
            initial[field.name] = getattr(attendee, field.name)

        return initial

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data

        user.attendee  # We should never be creating, here
        user.attendee, created = Attendee.objects.update_or_create(
            user=user, defaults=data)
        return super().form_valid(form)



from collections import OrderedDict

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from debconf.models import GENDERS

from debconf.tz import aoe_datetime
from register.fields import OptionalCountries


def build_fees():
    fees = []
    for id_, details in sorted(settings.PRICES['fee'].items(),
                               key=lambda fee: fee[1]['price']):
        price = details['price']
        if price:
            description = '{name} - {billing_price} {billing_currency}'
            if (settings.DEBCONF_BILLING_CURRENCY
                    != settings.DEBCONF_LOCAL_CURRENCY):
                description += ' or {local_price} {local_currency}'
        else:
            description = '{name} - Free'
        description = description.format(
            name=details['name'],
            billing_price=price,
            billing_currency=settings.DEBCONF_BILLING_CURRENCY,
            local_price=price * settings.DEBCONF_LOCAL_CURRENCY_RATE,
            local_currency=settings.DEBCONF_LOCAL_CURRENCY,
        )
        fees.append((id_, description))
    return OrderedDict(fees)


class Attendee(models.Model):
    FEES = build_fees()
    GENDERS = GENDERS

    user = models.OneToOneField(settings.AUTH_USER_MODEL,
                                related_name='attendee',
                                on_delete=models.PROTECT)

    # Contact information
    nametag_2 = models.CharField(max_length=50, blank=True)
    nametag_3 = models.CharField(max_length=50, blank=True)
    emergency_contact = models.TextField(blank=True)
    announce_me = models.BooleanField()
    register_announce = models.BooleanField()
    register_discuss = models.BooleanField()

    # Conference details
    coc_ack = models.BooleanField(default=False)
    fee = models.CharField(max_length=5, blank=True)
    arrival = models.DateTimeField(null=True, blank=True)
    departure = models.DateTimeField(null=True, blank=True)
    final_dates = models.BooleanField(default=False)
    reconfirm = models.BooleanField(default=False)

    # Personal information
    t_shirt_size = models.CharField(max_length=8, blank=True)
    shoe_size = models.CharField(max_length=8, blank=True)
    gender = models.CharField(max_length=1, blank=True)
    country = models.CharField(max_length=2, blank=True)
    languages = models.CharField(max_length=50, blank=True)
    pgp_fingerprints = models.TextField(blank=True)

    # COVID-19
    vaccinated = models.BooleanField(null=True)
    vaccination_notes = models.TextField(blank=True)
    confirm_covid_tests = models.BooleanField(null=True)

    # Billing
    invoiced_entity = models.TextField(blank=True)
    billing_address = models.TextField(blank=True)

    # Misc
    notes = models.TextField(blank=True)
    completed_register_steps = models.IntegerField(default=0)
    created_timestamp = models.DateTimeField(auto_now_add=True)
    updated_timestamp = models.DateTimeField(auto_now=True)
    completed_timestamp = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return 'Attendee <{}>'.format(self.user.username)

    def billable(self):
        """Is this user billable? (or paid)"""
        from bursary.models import Bursary

        try:
            bursary = self.user.bursary
        except Bursary.DoesNotExist:
            bursary = Bursary()

        if self.fee:
            return True

        try:
            if (self.food.meals.exists()
                    and not bursary.potential_bursary('food')):
                return True
        except ObjectDoesNotExist:
            pass

        try:
            if (self.accomm.nights.exists()
                    and not bursary.potential_bursary('accommodation')):
                return True
        except ObjectDoesNotExist:
            pass

        return False

    billable.boolean = True

    def paid(self):
        from invoices.prices import invoice_user
        invoices = self.user.invoices
        if invoices.filter(status='new').exists():
            return False

        invoice = invoice_user(self.user)
        return invoice['total'] <= 0

    paid.boolean = True

    def registered_before_deadline(self):
        if not self.completed_timestamp:
            return False
        return (
            self.completed_timestamp <
            aoe_datetime(settings.DEBCONF_CONFIRMATION_DEADLINE))

    registered_before_deadline.boolean = True

    @property
    def new_invoices(self):
        return self.user.invoices.filter(status='new')

    @property
    def keysigning_id(self):
        from register.models.queue import QueueSlot
        try:
            keysigning_slot = QueueSlot.objects.get(
                queue__name='PGP Keysigning', attendee=self)
            return keysigning_slot.monotonic_position
        except QueueSlot.DoesNotExist:
            return None

    @property
    def arrived(self):
        try:
            self.check_in
            return True
        except ObjectDoesNotExist:
            return False

    @property
    def country_name(self):
        return OptionalCountries().name(self.country)

    @property
    def shipping_address(self):
        from register.models.address import Address
        try:
            return Address.objects.get(attendee=self, role='shipping')
        except Address.DoesNotExist:
            return None

    def save(self, *args, **kwargs):
        if self.arrival == '':
            self.arrival = None
        if self.departure == '':
            self.departure = None
        return super().save(*args, **kwargs)



from pathlib import Path
from django.conf import global_settings

from debconf.common_settings import *

INSTALLED_APPS = (
    'minidebconf',
    'debconf',
) + INSTALLED_APPS

WAFER_REGISTRATION_MODE = 'custom'
WAFER_USER_IS_REGISTERED = 'minidebconf.models.is_registered'

ROOT_URLCONF = 'minidebconf.urls'

basedir = Path(__file__).parent.absolute()

MINIDEBCONF_REGISTER_PHONE = None

# manipulate middleware
MIDDLEWARE = list(MIDDLEWARE)

# i18n/l10n
session_idx = MIDDLEWARE.index("django.contrib.sessions.middleware.SessionMiddleware")
MIDDLEWARE.insert(session_idx, 'django.middleware.locale.LocaleMiddleware')
USE_I18N = True
USE_L10N = True
__languages__ = ['en'] + [p.name.replace('_', '-').lower()  for p in (basedir / 'locale').glob('*')]
LANGUAGES = [l for l in global_settings.LANGUAGES if l[0] in __languages__]



from django.contrib import admin
from django.contrib.admin.templatetags.admin_list import _boolean_icon

from register.admin.core import NotNullFilter
from register.models.food import Food


class SpecialDietNotNullFilter(NotNullFilter):
    title = "Special Diet"
    parameter_name = "special_diet"


class FoodAdmin(admin.ModelAdmin):
    list_display = (
        'attendee', 'full_name', 'email', 'diet', 'reconfirm', 'bursary'
    )
    list_filter = (
        'attendee__reconfirm', 'meals', 'diet',
        SpecialDietNotNullFilter,
        'attendee__user__bursary__food_status',
    )
    search_fields = (
        'attendee__user__username', 'attendee__user__first_name',
        'attendee__user__last_name'
    )

    def full_name(self, instance):
        return instance.attendee.user.get_full_name()
    full_name.admin_order_field = 'attendee__user__last_name'

    def email(self, instance):
        return instance.attendee.user.email
    email.admin_order_field = 'attendee__user__email'

    def bursary(self, instance):
        return instance.attendee.user.bursary.food_status
    bursary.short_description = 'Food bursary status'
    bursary.admin_order_field = 'attendee__user__bursary__food_status'

    def reconfirm(self, instance):
        return _boolean_icon(instance.attendee.reconfirm)
    reconfirm.short_description = 'Confirmed?'
    reconfirm.admin_order_field = 'attendee__reconfirm'


admin.site.register(Food, FoodAdmin)



from wafer.schedule.models import Slot
from debconf.views import get_current_slot
from datetime import timedelta


class TestGetCurrentSlot:
    def test_explicit_start_time(self, db, now):
        slot = Slot.objects.create(
            start_time=now - timedelta(minutes=30), end_time=now + timedelta(minutes=30)
        )
        assert get_current_slot() == slot

    def test_start_time_from_previous_slot(self, db, now):
        previous = Slot.objects.create(
            start_time=now - timedelta(hours=1), end_time=now
        )
        current = Slot.objects.create(
            previous_slot=previous, end_time=now + timedelta(hours=1)
        )
        assert get_current_slot() == current



# Generated by Django 2.2.20 on 2021-05-29 15:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0010_attendee_shipping_address'),
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.CharField(choices=[('billing', 'Billing'), ('shipping', 'Shipping')], max_length=32)),
                ('contact', models.CharField(max_length=50)),
                ('company', models.CharField(blank=True, max_length=50)),
                ('line1', models.CharField(max_length=50)),
                ('line2', models.CharField(blank=True, max_length=50)),
                ('city', models.CharField(max_length=50)),
                ('province', models.CharField(max_length=50)),
                ('postal_code', models.CharField(blank=True, max_length=32)),
                ('country', models.CharField(max_length=2)),
                ('phone_number', models.CharField(blank=True, max_length=32)),
                ('attendee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='addresses', to='register.Attendee')),
            ],
            options={
                'unique_together': {('attendee', 'role')},
            },
        ),
        migrations.RemoveField(
            model_name='attendee',
            name='shipping_address',
        ),
    ]
